from datetime import datetime, timedelta
from collections import namedtuple


def get_frontend_vals():
	'''Returns a named tuple of dynamic date ranges that match Redash's front-end
	'''

	ranges = calculate_ranges()
	singles = calculate_singletons()

	valkeys = [k for k in ranges.keys()] + [k for k in singles.keys()]

	Values = namedtuple('Values', ' '.join(valkeys))

	return Values(**ranges, **singles)

def calculate_ranges():

	# SuperToday is more specific than datetime.datetime.today
	SuperToday = namedtuple('SuperToday', 'month day year weeknum weekday')
	DateRange = namedtuple('DateRange', 'start end')

	today = datetime.today()
	_ymd = (today.month, today.day)

	t = SuperToday(*_ymd, *today.isocalendar())

	ranges = {}

	#  _____ _     _      __        __        _     
	# |_   _| |__ (_)___  \ \      / /__  ___| | __ 
	#   | | | '_ \| / __|  \ \ /\ / / _ \/ _ \ |/ / 
	#   | | | | | | \__ \   \ V  V /  __/  __/   <  
	#   |_| |_| |_|_|___/    \_/\_/ \___|\___|_|\_\ 

	start = datetime.strptime(f"{t.year}-{t.weeknum}-1", '%G-%V-%u')
	end = start + timedelta(days=6)

	ranges['d_this_week'] = DateRange(start,end)

											
	#  _____ _     _       __  __             _   _      
	# |_   _| |__ (_)___  |  \/  | ___  _ __ | |_| |__   
	#   | | | '_ \| / __| | |\/| |/ _ \| '_ \| __| '_ \  
	#   | | | | | | \__ \ | |  | | (_) | | | | |_| | | | 
	#   |_| |_| |_|_|___/ |_|  |_|\___/|_| |_|\__|_| |_| 

	start = datetime.strptime(f"{t.year}-{t.month}-1", '%Y-%m-%d')
	e_year, e_month = (t.year,t.month+1) if t.month < 12 else (t.year+1,1)
	end = datetime.strptime(
		f"{e_year}-{e_month}-1", '%Y-%m-%d') \
		- timedelta(days=1)

	ranges['d_this_month'] = DateRange(start,end)
											  
	#  _____ _     _      __   __                    
	# |_   _| |__ (_)___  \ \ / /__  __ _ _ __       
	#   | | | '_ \| / __|  \ V / _ \/ _` | '__|      
	#   | | | | | | \__ \   | |  __/ (_| | |         
	#   |_| |_| |_|_|___/   |_|\___|\__,_|_|         

	start = datetime.strptime(f"{t.year}-1-1", '%Y-%m-%d')
	end = datetime.strptime(f"{t.year}-12-31", '%Y-%m-%d')

	ranges['d_this_year'] = DateRange(start,end)

												  
	#  _              _    __        __        _       
	# | |    __ _ ___| |_  \ \      / /__  ___| | __   
	# | |   / _` / __| __|  \ \ /\ / / _ \/ _ \ |/ /   
	# | |__| (_| \__ \ |_    \ V  V /  __/  __/   <    
	# |_____\__,_|___/\__|    \_/\_/ \___|\___|_|\_\   

	start = datetime.strptime(f"{t.year}-{t.weeknum-1}-1", '%G-%V-%u')
	end = start + timedelta(days=6)

	ranges['d_last_week'] = DateRange(start,end)


	#  _              _     __  __             _   _      
	# | |    __ _ ___| |_  |  \/  | ___  _ __ | |_| |__   
	# | |   / _` / __| __| | |\/| |/ _ \| '_ \| __| '_ \  
	# | |__| (_| \__ \ |_  | |  | | (_) | | | | |_| | | | 
	# |_____\__,_|___/\__| |_|  |_|\___/|_| |_|\__|_| |_| 


	s_year, s_month = (t.year-1, 12) if t.month == 1 else (t.year,t.month-1)

	start = datetime.strptime(f"{s_year}-{s_month}-1", '%Y-%m-%d')
	end = datetime.strptime(f"{t.year}-{t.month}-1", '%Y-%m-%d') \
		- timedelta(days=1)

	ranges['d_last_month'] = DateRange(start, end)

	#  _              _    __   __                   
	# | |    __ _ ___| |_  \ \ / /__  __ _ _ __      
	# | |   / _` / __| __|  \ V / _ \/ _` | '__|     
	# | |__| (_| \__ \ |_    | |  __/ (_| | |        
	# |_____\__,_|___/\__|   |_|\___|\__,_|_|        

	start = datetime.strptime(f"{t.year-1}-1-1", '%Y-%m-%d')
	end = datetime.strptime(f"{t.year-1}-12-31", '%Y-%m-%d')

	ranges['d_last_year'] = DateRange(start,end)


	 #  _              _    __  __  ____                   
	 # | |    __ _ ___| |_  \ \/ / |  _ \  __ _ _   _ ___  
	 # | |   / _` / __| __|  \  /  | | | |/ _` | | | / __| 
	 # | |__| (_| \__ \ |_   /  \  | |_| | (_| | |_| \__ \ 
	 # |_____\__,_|___/\__| /_/\_\ |____/ \__,_|\__, |___/ 
	 #                                          |___/      

	def make_x_days_date_range(today, days):
		start = today - timedelta(days=days)
		end = today

		return DateRange(start, end)

	for x in [7, 14, 30, 60, 90]:
		ranges[f"d_last_{x}_days"] = make_x_days_date_range(today, x)

	return ranges

def calculate_singletons():

	today = datetime.today()
	d_now=datetime.strptime(
		f"{today.year}-{today.month}-{today.day}", '%Y-%m-%d')
	d_yesterday = d_now - timedelta(days=1)

	return dict(d_now=d_now, d_yesterday=d_yesterday)



import click
import requests

template = u"""/*
Name: {name}
Data source: {data_source}
Created By: {created_by}
Last Update At: {last_updated_at}
*/
{query}"""


def get_queries(url, api_key):
    queries = [] 
    headers = {'Authorization': 'Key {}'.format(api_key)}
    path = "{}/api/queries".format(url)
    has_more = True
    page = 1
    while has_more:
        response = requests.get(path, headers=headers, params={'page': page}).json()
        queries.extend(response['results'])
        has_more = page * response['page_size'] + 1 <= response['count']
        page += 1

    return queries


def save_queries(queries):
    for query in queries:
        filename = 'query_{}.sql'.format(query['id'])
        with open(filename, 'w') as f:
            content = template.format(name=query['name'],
                       data_source=query['data_source_id'],
                       created_by=query['user']['name'],
                       last_updated_at=query['updated_at'],
                       query=query['query'])
            f.write(content)


@click.command()
@click.option('--redash-url')
@click.option('--api-key', help="API Key")
def main(redash_url, api_key):
    queries = get_queries(redash_url, api_key)      
    save_queries(queries)


if __name__ == '__main__':
    main()


import click
from redash_toolbelt import Redash, get_frontend_vals


def refresh_dashboard(baseurl, apikey, slug):

    client = Redash(baseurl, apikey)
    todays_dates = get_frontend_vals()
    queries_dict = get_queries_on_dashboard(client, slug)

    # loop through each query and its JSON data
    for idx, qry in queries_dict.items():

        params = {
            p.get("name"): fill_dynamic_val(todays_dates, p)
            for p in qry["options"].get("parameters", [])
        }

        # Pass max_age to ensure a new result is provided.
        body = {"parameters": params, "max_age": 0}

        r = client._post(f"api/queries/{idx}/results", json=body)

        print(f"Query: {idx} -- Code {r.status_code}")


def get_queries_on_dashboard(client, slug):

    # Get a list of queries on this dashboard
    dash = client.dashboard(slug=slug)

    # Dashboards have visualization and text box widgets. Get the viz widgets.
    viz_widgets = [i for i in dash["widgets"] if "visualization" in i.keys()]

    # Visualizations are tied to queries
    l_query_ids = [i["visualization"]["query"]["id"] for i in viz_widgets]

    return {id: client._get(f"api/queries/{id}").json() for id in l_query_ids}


def fill_dynamic_val(dates, p):
    """Accepts parameter default information from the Redash API.

    If the default value is not a date type, or its value cannot be calculated,
    then the default value is returned unchanged.

	Otherwise, the dynamic value is retrieved from the dates param and returned
	"""

    if not is_dynamic_param(dates, p):
        return p.get("value")
    dyn_val = getattr(dates, p.get("value"))

    if is_date_range(dyn_val):
        return format_date_range(dyn_val)
    else:
        return format_date(dyn_val)


def is_dynamic_param(dates, param):

    return "date" in param.get("type") and param.get("value") in dates._fields


def is_date_range(value):
    return hasattr(value, "start") and hasattr(value, "end")


def format_date(date_obj):
    return date_obj.strftime("%Y-%m-%d")


def format_date_range(date_range_obj):

    start = format_date(date_range_obj.start)
    end = format_date(date_range_obj.end)

    return dict(start=start, end=end)


@click.command()
@click.argument("url",)
@click.argument("key",)
@click.argument("slug",)
def main(url, key, slug):
    """Refresh URL/dashboards/SLUG using KEY"""

    refresh_dashboard(url, key, slug)


if __name__ == "__main__":
    main()



import click

from redash_toolbelt import Redash


class Lookup(object):
    def __init__(self, redash, email):
        self.email = email.lower()
        self.redash = redash

    def check_query_result(self, query_result_id):
        if not query_result_id:
            return False

        result = self.redash._get(
            "api/query_results/{}".format(query_result_id))

        return self.email in result.text.lower()

    def check_query(self, query):
        found_in_query = False
        for field in ("query", "name", "description"):
            if self.email in (query[field] or "").lower():
                found_in_query = True

        for tag in query["tags"]:
            if self.email in (tag or "").lower():
                found_in_query = True

        found_in_result = self.check_query_result(
            query["latest_query_data_id"])

        return found_in_query or found_in_result

    def check_dashboard(self, dashboard):

        found_in_dashboard = False
        found_in_widget = False

        for field in ("slug", "name"):
            if self.email in (dashboard[field] or "").lower():
                found_in_dashboard = True

        for tag in dashboard["tags"]:
            if self.email in (tag or "").lower():
                found_in_dashboard = True

        if not found_in_dashboard:
            dash_widgets = self.redash._get("api/dashboards/{}".format(
                dashboard["slug"])).json()["widgets"]

            # Check text widgets
            if not isinstance(dash_widgets, type(None)):
                for widget in dash_widgets:
                    if "visualization" not in widget and self.email in widget[
                            "text"]:
                        found_in_widget = True

        return found_in_dashboard or found_in_widget

    def lookup(self):
        queries = self.redash.paginate(self.redash.queries)

        with click.progressbar(queries, label="Queries") as bar:
            found_q = [query for query in bar if self.check_query(query)]

        for query in found_q:
            query_url = "{}/queries/{}".format(self.redash.redash_url,
                                               query["id"])
            print(query_url)

        dashboards = self.redash.paginate(self.redash.dashboards)

        with click.progressbar(dashboards, label="Dashboards") as bar:
            found_d = [dash for dash in bar if self.check_dashboard(dash)]

        for dash in found_d:
            dash_url = "{}/dashboards/{}".format(self.redash.redash_url,
                                                 dash["slug"])
            print(dash_url)


@click.command()
@click.argument("redash_host")
@click.argument("email")
@click.option(
    "--api-key",
    "api_key",
    envvar="REDASH_API_KEY",
    show_envvar=True,
    prompt="API Key",
    help="User API Key",
)
def lookup(redash_host, email, api_key):
    """Search for EMAIL in queries and query results, output query URL if found."""

    redash = Redash(redash_host, api_key)
    lookup = Lookup(redash, email)
    lookup.lookup()


if __name__ == "__main__":
    lookup()



import os
import requests
import time
from pprint import pprint
import json


def poll_job(s, redash_url, job):
    # TODO: add timeout
    while job['status'] not in (3,4):
        response = s.get('{}/api/jobs/{}'.format(redash_url, job['id']))
        job = response.json()['job']
        time.sleep(1)

    if job['status'] == 3:
        return job['query_result_id']
    
    return None


def get_fresh_query_result(redash_url, query_id, api_key, params):
    s = requests.Session()
    s.headers.update({'Authorization': 'Key {}'.format(api_key)})

    payload = dict(max_age=0, parameters=params)

    response = s.post('{}/api/queries/{}/results'.format(redash_url, query_id), data=json.dumps(payload))

    if response.status_code != 200:
        raise Exception('Refresh failed.')

    result_id = poll_job(s, redash_url, response.json()['job'])

    if result_id:
        response = s.get('{}/api/queries/{}/results/{}.json'.format(redash_url, query_id, result_id))
        if response.status_code != 200:
            raise Exception('Failed getting results.')
    else:
        raise Exception('Query execution failed.')

    return response.json()['query_result']['data']['rows']

if __name__ == '__main__':
    params = {'some_parameter': 1}
    query_id = 1234
    # Need to use a *user API key* here (and not a query API key).
    api_key = '...'
    pprint(get_fresh_query_result('https://app.redash.io/acme', query_id, api_key, params))


import requests


class Redash(object):
    def __init__(self, redash_url, api_key):
        self.redash_url = redash_url
        self.session = requests.Session()
        self.session.headers.update({'Authorization': 'Key {}'.format(api_key)})

    def test_credentials(self):
        try:
            response = self._get('api/session')
            return True
        except requests.exceptions.HTTPError:
            return False

    def queries(self, page=1, page_size=25):
        """GET api/queries"""
        return self._get('api/queries', params=dict(page=page, page_size=page_size)).json()

    def dashboards(self, page=1, page_size=25):
        """GET api/dashboards"""
        return self._get('api/dashboards', params=dict(page=page, page_size=page_size)).json()

    def dashboard(self, slug):
        """GET api/dashboards/{slug}"""
        return self._get('api/dashboards/{}'.format(slug)).json()

    def create_dashboard(self, name):
        return self._post('api/dashboards', json={'name': name}).json()

    def update_dashboard(self, dashboard_id, properties):
        return self._post('api/dashboards/{}'.format(dashboard_id), json=properties).json()

    def create_widget(self, dashboard_id, visualization_id, text, options):
        data = {
            'dashboard_id': dashboard_id,
            'visualization_id': visualization_id,
            'text': text,
            'options': options,
            'width': 1,
        }
        return self._post('api/widgets', json=data)

    def duplicate_dashboard(self, slug, new_name=None):
        current_dashboard = self.dashboard(slug)

        if new_name is None:
            new_name = u'Copy of: {}'.format(current_dashboard['name'])

        new_dashboard = self.create_dashboard(new_name)
        if current_dashboard['tags']:
            self.update_dashboard(new_dashboard['id'], {'tags': current_dashboard['tags']})

        for widget in current_dashboard['widgets']:
            visualization_id = None
            if 'visualization' in widget:
                visualization_id = widget['visualization']['id']
            self.create_widget(new_dashboard['id'], visualization_id, widget['text'], widget['options'])

        return new_dashboard

    def scheduled_queries(self):
        """Loads all queries and returns only the scheduled ones."""
        queries = self.paginate(self.queries)
        return filter(lambda query: query['schedule'] is not None, queries)

    def update_query(self, query_id, data):
        """POST /api/queries/{query_id} with the provided data object."""
        path = 'api/queries/{}'.format(query_id)
        return self._post(path, json=data)

    def paginate(self, resource):
        """Load all items of a paginated resource"""
        stop_loading = False
        page = 1
        page_size = 100

        items = []

        while not stop_loading:
            response =  resource(page=page, page_size=page_size)

            items += response['results']
            page += 1

            stop_loading = response['page'] * response['page_size'] >= response['count']

        return items

    def _get(self, path, **kwargs):
        return self._request('GET', path, **kwargs)

    def _post(self, path, **kwargs):
        return self._request('POST', path, **kwargs)

    def _request(self, method, path, **kwargs):
        url = '{}/{}'.format(self.redash_url, path)
        response = self.session.request(method, url, **kwargs)
        response.raise_for_status()
        return response
